/*	AudioGUI.java
 *	Author: Jimmy Yih, Mark Crompton 2011
 *	
 *	This is the Graphical User Interface of the XRMB/MOCHA Demo.
 */

import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.TargetDataLine;
import javax.sound.sampled.UnsupportedAudioFileException;
import javax.swing.UIManager;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

// Import the Matlab packages
import Spectrogram.Spectrogram;
import XRMB.XRMB;
import MOCHA.MOCHA;

import com.mathworks.toolbox.javabuilder.MWClassID;
import com.mathworks.toolbox.javabuilder.MWComplexity;
import com.mathworks.toolbox.javabuilder.MWException;
import com.mathworks.toolbox.javabuilder.MWNumericArray;


public class AudioGUI implements ActionListener{
	public JFrame frame;
	public JPanel panel;
	public ImagePanel specPanel;
	public JPanel bigPanel;
	public AudioManager.WaveformPanel wavPane;
	public JButton record;
	public JButton play;
	public JButton pause;
	public JButton load;
	public JButton save;
	public JMenuBar menubar;
	public JMenu menu_file;
	public JMenu menu_about;
	public JMenu menu_help;
	public JMenuItem mt_about;
	public JMenuItem mt_help;
	public JMenuItem mt_quit;
	public JCheckBox checkSave;
	public JComboBox combo;
	public JComboBox choiceCombo;
	public TickMarkPanel tmp;
	public int state;
	public boolean canceled = false;
	private boolean recording = false;

	// JComboBox combo states
	public static final int _RECORD_ = 0;
	public static final int _XRMB_ = 1;
	public static final int _MOCHA_ = 2;

	StringBuffer stringbuff_help;
	StringBuffer stringbuff_about;
	AudioInputStream audioInputStream;
	AudioInputStream loadInputStream;
	Spectrogram s = null;
	XRMB x = null;
	MOCHA m = null;
	String file;
	String demoFile;
	String pha;
	String pal;
	String movie;
	TargetDataLine line;
	byte[] data;
	AudioFormat format;
	public AudioGUI(){
		frame = new JFrame("XRMB/MOCHA Demo");
		panel = new JPanel();
		specPanel = new ImagePanel();
		bigPanel = new JPanel();
		wavPane = new AudioManager.WaveformPanel();
		load = new JButton("Load");
		save = new JButton("Save");
		record = new JButton("Record");
		play = new JButton("Play");
		pause = new JButton("Pause");
		menubar = new JMenuBar();
		menu_file = new JMenu("File");
		menu_about = new JMenu("About");
		menu_help = new JMenu("Help");
		mt_help = new JMenuItem("Help");
		mt_about = new JMenuItem("About");
		mt_quit = new JMenuItem("Quit");
		checkSave = new JCheckBox("Save");
		checkSave.setVisible(false);

		// Sets up the graph for the Spectrogram
		tmp = new TickMarkPanel(0, 2.5f, 0, 8, 6, 5);
		tmp.setXLabel("Time (secs)");
		tmp.setYLabel("Frequency (kHz)");

		// Sets up first combo box
		combo = new JComboBox();
		combo.setPreferredSize(new Dimension(100, 10));

		// Sets up second combo box
		choiceCombo = new JComboBox();
		choiceCombo.setPreferredSize(new Dimension(100, 10));
		choiceCombo.setVisible(false);
		state = _RECORD_;

		stringbuff_help = new StringBuffer();
		stringbuff_about = new StringBuffer();
		BufferedReader reader = null;
		String text = null;
		movie = "default.mov";

		// Reads in Help file
		try{
			reader = new BufferedReader(new FileReader("txt/Help"));

			while((text = reader.readLine()) != null)
				stringbuff_help.
				append(text).
				append(System.getProperty("line.separator"));
			reader.close();
		}
		catch(Exception e){

		}

		text = null;

		// Reads in About file
		try{
			reader = new BufferedReader(new FileReader("txt/About"));

			while((text = reader.readLine()) != null)
				stringbuff_about.
				append(text).
				append(System.getProperty("line.separator"));
			reader.close();

		}
		catch(Exception e){

		}
		
		// Initialize Spectrogram object
		try {
			s = new Spectrogram();
		} catch (MWException e1) {
			e1.printStackTrace();
		}

		// Sets up the format of the Audio
		format = new AudioFormat(44100, 8, 1, true, true);
		System.out.println(format);
		try {
			line = AudioManager.getTargetDataLine(format);
		} catch (LineUnavailableException e) {
			e.printStackTrace();
		}

		init();
	}

	public void init(){
		// Add action listeners to each item
		record.addActionListener(this);
		play.addActionListener(this);
		pause.addActionListener(this);
		load.addActionListener(this);
		save.addActionListener(this);

		mt_quit.addActionListener(this);
		mt_help.addActionListener(this);
		mt_about.addActionListener(this);

		menu_file.add(mt_quit);
		menubar.add(menu_file);
		menu_help.add(mt_help);
		menubar.add(menu_help);
		menu_about.add(mt_about);
		menubar.add(menu_about);
		frame.setJMenuBar(menubar);

		combo.addItem("Record");
		combo.addItem("XRMB");
		combo.addItem("MOCHA");
		combo.addActionListener(this);

		choiceCombo.addItem("Load");
		choiceCombo.addItem("Demo 1");
		choiceCombo.addActionListener(this);

		checkSave.addActionListener(this);

		panel.setLayout(new GridLayout(1, 3));
		panel.add(combo);
		panel.add(choiceCombo);
		panel.add(checkSave);
		panel.add(load);
		panel.add(save);
		panel.add(record);
		panel.add(play);
		panel.add(pause);

		bigPanel.setLayout(new BoxLayout(bigPanel, BoxLayout.Y_AXIS));
		wavPane.setPreferredSize(new Dimension(10, 100));
		specPanel.setPreferredSize(new Dimension(10, 100));
		tmp.setPreferredSize(new Dimension(20, 200));
		bigPanel.add(panel);
		bigPanel.add(wavPane);
		tmp.add(specPanel);
		bigPanel.add(tmp);
		frame.add(bigPanel);
		frame.pack();
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	@Override
	public void actionPerformed(ActionEvent arg0) {
		if(arg0.getSource() == record){
			if(record.getText().equals("Record")){
				recording = true;
				// Recording Thread
				Thread t = new Thread(){
					public void run(){
						try {
							data = AudioManager.startRecording(line);
							recording = false;
						} catch (LineUnavailableException e) {
							e.printStackTrace();
						} catch (IOException e) {
							e.printStackTrace();
						}
						audioInputStream = AudioManager.getAudioInputStream(data, format);
						try {
							float seconds = (float)((audioInputStream.getFrameLength()) / audioInputStream.getFormat().getFrameRate());
							tmp.xHigh = seconds;
							tmp.xTicks = (int)(seconds * 2 + 1);
							if (tmp.xTicks > 20)
								tmp.xTicks = 20;
							wavPane.setLines(AudioManager.createWaveform(audioInputStream, wavPane.getSize()));
							audioInputStream.reset();
							double[][] spectrogram = AudioManager.createSpectrogram(audioInputStream);
							BufferedImage img = AudioManager.getSpectrogramImage(spectrogram);
							specPanel.setImage(img);
							tmp.repaint();
							
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
				};
				// dynamic waveform painting (may not work if toWaveform is null for many loops.. hence the sleep call)
				Thread dynWav = new Thread() {
					public void run() {
						try {
							while(recording) {
								byte[] toWaveform = AudioManager.output;
								if (toWaveform != null) {
									AudioInputStream audioStream = AudioManager.getAudioInputStream(toWaveform, format);
									wavPane.setLines(AudioManager.createWaveform(audioStream, wavPane.getSize()));
								}
								Thread.sleep(50);
							}
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
				};
				t.start();
				dynWav.start();
				record.setText("Stop");
				pause.setText("Pause");
			}
			else{
				AudioManager.stopRecording();
				record.setText("Record");
				file = null;
			}
		}
		else if(arg0.getSource() == play){
			if(play.getText().equals("Play") && data != null){
				play.setText("Stop");
				// Play Thread
				Thread t = new Thread(){
					public void run(){
						try {
							if(file == null){
								audioInputStream = AudioManager.getAudioInputStream(data, format);
								AudioManager.startPlayback(AudioManager.getAudioInputStream(data, audioInputStream.getFormat()));
							}
							else{
								loadInputStream = AudioManager.load(file);
								AudioManager.startPlayback(loadInputStream);
							}
						} catch (LineUnavailableException e) {
							e.printStackTrace();
						} catch (IOException e) {
							e.printStackTrace();
						} catch (UnsupportedAudioFileException e) {
							e.printStackTrace();
						}
						pause.setText("Pause");
						play.setText("Play");
					}
				};
				// Thread that creates vertical lines on the waveform
				Thread vert = new Thread() {
					public void run() {
						try {
							int milliseconds;
							if(file == null){
								audioInputStream = AudioManager.getAudioInputStream(data, format);
								milliseconds = (int)((audioInputStream.getFrameLength() * 1000) / audioInputStream.getFormat().getFrameRate());
							}
							else{
								loadInputStream = AudioManager.load(file);
								milliseconds = (int)((loadInputStream.getFrameLength() * 1000) / loadInputStream.getFormat().getFrameRate());
							}
							wavPane.VertLine(milliseconds);
						} catch (IOException e) {
							e.printStackTrace();
						} catch (UnsupportedAudioFileException e) {
							e.printStackTrace();
						}
					}
				};
				// Thread that runs XRMB
				Thread xrmb = new Thread() {
					public void run() {
						try {
							Object[] output = new Object[0];
							Object[] input = new Object[5];
							input[0] = demoFile;
							input[1] = pha;
							input[2] = pal;
							input[3] = checkSave.isSelected();
							input[4] = movie;
			
							x = new XRMB();
							x.XRMBdemo(output, input);
						} catch (MWException e) {
							e.printStackTrace();
						}
					}
				};
				// Thread that runs MOCHA
				Thread mocha = new Thread() {
					public void run() {
						try {
							Object[] output = new Object[0];
							Object[] input = new Object[3];
							input[0] = demoFile;
							input[1] = checkSave.isSelected();
							input[2] = movie;
			
							m = new MOCHA();
							m.MOCHAdemo(output, input);
						} catch (MWException e) {
							e.printStackTrace();
						}
					}
				};
				t.start();
				vert.start();
				if (state == _XRMB_)
					xrmb.start();
				else if (state == _MOCHA_)
					mocha.start();
			}
			else{
				play.setText("Play");
				AudioManager.stopPlayback();
			}
		}
		// Pause Button
		else if(arg0.getSource() == pause){
			if(pause.getText().equals("Pause")){
				pause.setText("Resume");
				AudioManager.pausePlayback();
			}
			else{
				pause.setText("Pause");
				AudioManager.resumePlayback();
			}
		}
		// Load Button
		else if(arg0.getSource() == load){
			File file = new File(System.getProperty("user.dir"));
			JFileChooser fc = new JFileChooser(file);
			fc.setFileFilter(new javax.swing.filechooser.FileFilter () {
				public boolean accept(File f) {
					if (f.isDirectory()) {
						return true;
					}
					String name = f.getName();
					if (name.endsWith(".au") || name.endsWith(".wav") || name.endsWith(".aiff") || name.endsWith(".aif")) {
						return true;
					}
					return false;
				}
				public String getDescription() {
					return ".au, .wav, .aif";
				}
			});

			if (fc.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
				this.file = fc.getSelectedFile().getAbsolutePath();
				loadFile(this.file);
			}
		}
		// Save Button for Record state
		else if(arg0.getSource() == save){
			if(data == null)
				return;
			audioInputStream = AudioManager.getAudioInputStream(data, format);
			try {
				File file = new File(System.getProperty("user.dir"));
				JFileChooser fc = new JFileChooser(file);
				if(fc.showSaveDialog(frame) == JFileChooser.APPROVE_OPTION)
					AudioManager.saveToWave(audioInputStream, fc.getSelectedFile().getAbsolutePath());
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		// Combo box that sets the state (Record/XRMB/MOCHA)
		else if(arg0.getSource() == combo)
		{
			if(combo.getSelectedItem().equals("Record"))
			{
				state = _RECORD_;
				choiceCombo.setVisible(false);
				checkSave.setVisible(false);
				load.setVisible(true);
				record.setVisible(true);
				save.setVisible(true);
				wavPane.setLines(null);
				wavPane.repaint();
				specPanel.setImage(null);
				specPanel.repaint();
			}
			else if(combo.getSelectedItem().equals("XRMB"))
			{
				state = _XRMB_;
				choiceCombo.setVisible(true);
				choiceCombo.setSelectedItem("Load");
				checkSave.setVisible(true);
				load.setVisible(false);
				record.setVisible(false);
				save.setVisible(false);
				wavPane.setLines(null);
				wavPane.repaint();
				specPanel.setImage(null);
				specPanel.repaint();
			}
			else if(combo.getSelectedItem().equals("MOCHA"))
			{
				state = _MOCHA_;
				choiceCombo.setVisible(true);
				choiceCombo.setSelectedItem("Load");
				checkSave.setVisible(true);
				load.setVisible(false);
				record.setVisible(false);
				save.setVisible(false);
				wavPane.setLines(null);
				wavPane.repaint();
				specPanel.setImage(null);
				specPanel.repaint();
			}
		}
		// Quit Menu Item
		else if(arg0.getSource() == mt_quit){
			SwingUtilities.invokeLater(new Runnable() {
	            public void run() {
	            	int option = JOptionPane.showConfirmDialog(
	            			new JFrame(),
	    					"Do you really want to quit?",
	    					"Quit",
	    					JOptionPane.YES_NO_OPTION);
	    			if(option == 0)
	    				System.exit(0);
	            }
	        });
		}
		// Combo box for choosing which demo to perform (Currently hardcoded)
		else if (arg0.getSource() == choiceCombo) {
			if (choiceCombo.getSelectedItem().equals("Demo 1")) {
				if (state == _XRMB_) {
					demoFile = "dat/jw11_tp105";
					pha = "dat/jw11_pha.dat";
					pal = "dat/jw11_pal.dat";

					file = demoFile +".wav";
					loadFile(file);
				}
				if (state == _MOCHA_) {
					demoFile = "dat/fsew0_001";

					file = demoFile +".wav";
					loadFile(file);
				}
			}
		}
		// Save check button for XRMB/MOCHA
		else if (arg0.getSource() == checkSave) {
			boolean selected = checkSave.isSelected();
			if (selected) {
				SwingUtilities.invokeLater(new Runnable() {
			        public void run() {
			        	Object result = JOptionPane.showInputDialog(
								new JFrame(),
								"Enter movie name:"
						);
						movie = ((String) result) +".mov";
			        }
			    });
			}
			else
				movie = "default.mov";
		}
		// Displays About file
		else if(arg0.getSource() == mt_about)
		{
			SwingUtilities.invokeLater(new Runnable() {
	            public void run() {
	            	JOptionPane.showMessageDialog(
	    					new JFrame(),
	    					stringbuff_about,
	    					"About",
	    					JOptionPane.INFORMATION_MESSAGE
	    			);
	            }
	        });
		}
		// Displays Help file
		else if(arg0.getSource() == mt_help)
		{
			SwingUtilities.invokeLater(new Runnable() {
	            public void run() {
	            	JOptionPane.showMessageDialog(
	    					new JFrame(),
	    					stringbuff_help,
	    					"Help",
	    					JOptionPane.QUESTION_MESSAGE
	    			);
	            }
	        });
		}
	}

	// loads audio and creates the waveform & spectrogram
	public void loadFile(String file_name) {
		try {
			loadInputStream = AudioManager.load(file_name);
			data = new byte[1];
			wavPane.setLines(AudioManager.createWaveform(loadInputStream, wavPane.getSize()));
			loadInputStream = AudioManager.load(file);
			float seconds = (float)((loadInputStream.getFrameLength()) / loadInputStream.getFormat().getFrameRate());
			tmp.xHigh = seconds;
			tmp.xTicks = (int)(seconds * 2 + 1);
			if (tmp.xTicks > 20)
				tmp.xTicks = 20;
			double[][] spectrogram = AudioManager.createSpectrogram(loadInputStream);
			BufferedImage img = AudioManager.getSpectrogramImage(spectrogram);
			specPanel.setImage(img);
			tmp.repaint();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static void main(String[] args){
		try {
			if(!UIManager.getSystemLookAndFeelClassName().equals("com.sun.java.swing.plaf.windows.WindowsLookAndFeel"))
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			e.printStackTrace();
		}
		AudioGUI a = new AudioGUI();
	}
}
