//
//  LoadImage.java
//  
//
//  Created by David King on 3/25/10.
//	Edited by Mark Crompton and Jimmy Yih on 5/17/11
//  Copyright 2010 Miguel A. Carreira-Perpinan. All rights reserved.
//

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.awt.image.IndexColorModel;
import java.awt.image.Raster;
import java.awt.image.WritableRaster;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.JPanel;

public class ImagePanel extends JPanel{
	private static final long serialVersionUID = 1L;
	//This is our global image
	private BufferedImage img;
	private double zoomWidth;
	private double zoomHeight;
	WritableRaster wr;

	//This pains our Component with the graphics given (our image)
	public void paint(Graphics g)
	{
		if(img != null){
			int width = (int) (img.getWidth()*zoomWidth);
			int height = (int) (img.getHeight()*zoomHeight);
			g.drawImage(img,0,0,width,height,null);
		}
		else
			super.paint(g);
	}

	//Default Constructor
	public ImagePanel()
	{
		img = null;
		zoomWidth = 1;
		zoomHeight = 1;
	}

	//This constructor takes in a BufferedImage
	public ImagePanel(BufferedImage input, double zw, double zh)
	{
		img = input;
		zoomWidth = zw;
		zoomHeight = zh;
	}
	
	//This sets our preferredsize of our component
	public Dimension getPreferredSize()
	{
		if(img == null)
			return new Dimension(256,256);
		else
			return new Dimension((int)(img.getWidth(null)*zoomWidth),(int)(img.getHeight(null)*zoomHeight));
	}
	
	public void setSize(int width, int height){
		super.setSize(width, height);
		zoomWidth = (double) this.getWidth() / (double) img.getWidth();
		zoomHeight = (double) this.getHeight() / (double) img.getHeight();
	}

	//We can get our image from this
	public BufferedImage getImage()
	{
		return img;
	}
	
	public void changeZoom(double zw, double zh){
		zoomWidth = zw;
		zoomHeight = zh;
	}
	
	public double getZoomWidth(){
		return zoomWidth;
	}
	
	public double getZoomHeight(){
		return zoomHeight;
	}

	public int getMSWidth()
	{
		return img.getData().getWidth();
	}

	public int getMSHeight()
	{
		return img.getData().getHeight();
	}

	//We can set our image from this
	public void setImage(BufferedImage input)
	{
		img = input;
		if (input == null)
			return;
		zoomWidth = (double) this.getWidth() / (double) input.getWidth();
		zoomHeight = (double) this.getHeight() / (double) input.getHeight();
		validate();
		repaint();
	}
}
