/*	TickMarkPanel.java
 *	Author: Jimmy Yih, Mark Crompton 2011
 *	
 *	Used to display Spectrogram in graphical form.
 */

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;

import javax.swing.BoxLayout;
import javax.swing.JPanel;


public class TickMarkPanel extends JPanel {
	private static final long serialVersionUID = 4426483663727836349L;
	protected int SPACE = 15;
	protected int YBUFFER = 40;
	protected int XBUFFER = 40;
	protected int TICK = 5;
	protected float xLow, xHigh, yLow, yHigh;
	protected int xTicks, yTicks;
	protected String xLabel = "";
	protected String yLabel = "";
	
	public TickMarkPanel(float xl, float xh, float yl, float yh, int xt, int yt) {
		super();
		xLow = xl;
		xHigh = xh;
		yLow = yl;
		yHigh = yh;
		xTicks = xt;
		yTicks = yt;
		this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
	}
	
	public void paint(Graphics arg0) {
		super.paintComponent(arg0);
		Graphics2D g = (Graphics2D) arg0;
		g.setFont(new Font("Arial", Font.BOLD, 12));
		g.setColor(Color.BLACK);
		Component c = this.getComponent(0);
		int height = this.getHeight();
		int width = this.getWidth();
		if(c != null)
			c.repaint(YBUFFER, SPACE, width-SPACE-YBUFFER, height-SPACE-YBUFFER);
		// X-Axis
		g.drawLine(YBUFFER, height-XBUFFER, width-SPACE, height-XBUFFER);
		// Y-Axis
		g.drawLine(YBUFFER, SPACE, YBUFFER, height-XBUFFER);
		// X-Axis Ticks
		float xSpace = (float) (width-SPACE-YBUFFER) / (float) (xTicks-1);
		float xInter = (xHigh-xLow) / (xTicks-1);
		for(int x = 0; x < xTicks; x++){
			int xx = (int) (YBUFFER+xSpace*x);
			g.drawLine(xx, height-XBUFFER, xx, height-XBUFFER+TICK);
			float val = (int)((xLow+xInter*x)*10) / 10.0f;
			String num = Float.toString(val);
			int offset = g.getFontMetrics().stringWidth(num) / 2;
			g.drawString(num, xx-offset, height-XBUFFER+TICK*3+1);
		}
		int xCenter = (int) ((width-SPACE-YBUFFER)/2.0f + YBUFFER);
		int xLabLen = g.getFontMetrics().stringWidth(xLabel);
		g.drawString(xLabel, xCenter-xLabLen/2, height-XBUFFER+TICK*6);
		// Y-Axis Ticks
		float ySpace = (float) (height-SPACE-XBUFFER) / (float) (yTicks-1);
		float yInter = (yHigh-yLow) / (yTicks-1);
		for(int y = 0; y < yTicks; y++){
			int yy = (int) ((height-XBUFFER)-ySpace*y);
			g.drawLine(YBUFFER, yy, YBUFFER-TICK, yy);
			float val = (int)((yLow+yInter*y)*10) / 10.0f;
			String num = Float.toString(val);
			int xOffset = g.getFontMetrics().stringWidth(num)+1;
			int yOffset = g.getFontMetrics().getHeight()/3;
			g.drawString(num, YBUFFER-TICK-xOffset, yy+yOffset);
		}
		int yCenter = (int) ((height-XBUFFER-SPACE)/2.0f + SPACE);
		int yLabLen = g.getFontMetrics().stringWidth(yLabel);
		g.rotate(Math.PI/2.0f, TICK, yCenter-yLabLen/2);
		g.drawString(yLabel, TICK, yCenter-yLabLen/2);
	}
	
	public Component add(Component c) {
		Component[] comps = this.getComponents();
		if(comps.length == 0)
			return super.add(c);
		return null;
	}
	
	public void setXLabel(String xLabel) {
		this.xLabel = xLabel;
	}
	
	public void setYLabel(String yLabel) {
		this.yLabel = yLabel;
	}
}
